@@ -1024,23 +1024,113 @@ void cmd_mode(I3_CMD, const char *mode) {
 }
 
 /*
- * Implementation of 'move [window|container] [to] output <str>'.
+ * Implementation of 'move [window|container|workspace] [to] output <strings>'.
  *
  */
-void cmd_move_con_to_output(I3_CMD, const char *name) {
-    DLOG("Should move window to output \"%s\".\n", name);
-    HANDLE_EMPTY_MATCH;
+void cmd_move_con_to_output(I3_CMD, const char *name, bool move_workspace) {
+    /* Initialize a data structure that is used to save multiple user-specified
+     * output names since this function is called multiple types for each
+     * command call. */
+    typedef struct user_output_name {
+        char *name;
+        TAILQ_ENTRY(user_output_name) user_output_names;
+    } user_output_name;
+    static TAILQ_HEAD(user_output_names_head, user_output_name) user_output_names = TAILQ_HEAD_INITIALIZER(user_output_names);
 
-    owindow *current;
-    bool had_error = false;
-    TAILQ_FOREACH (current, &owindows, owindows) {
-        DLOG("matching: %p / %s\n", current->con, current->con->name);
+    if (name) {
+        if (strcmp(name, "next") == 0) {
+            /* "next" here works like a wildcard: It "expands" to all available
+             * outputs. */
+            Output *output;
+            TAILQ_FOREACH (output, &outputs, outputs) {
+                user_output_name *co = scalloc(sizeof(user_output_name), 1);
+                co->name = sstrdup(output_primary_name(output));
+                TAILQ_INSERT_TAIL(&user_output_names, co, user_output_names);
+            }
+            return;
+        }
 
-        had_error |= !con_move_to_output_name(current->con, name, true);
+        user_output_name *co = scalloc(sizeof(user_output_name), 1);
+        co->name = sstrdup(name);
+        TAILQ_INSERT_TAIL(&user_output_names, co, user_output_names);
+        return;
     }
 
-    cmd_output->needs_tree_render = true;
-    ysuccess(!had_error);
+    HANDLE_EMPTY_MATCH;
+
+    if (TAILQ_EMPTY(&user_output_names)) {
+        yerror("At least one output must be specified");
+        return;
+    }
+
+    bool success = false;
+    user_output_name *uo;
+    owindow *current;
+    TAILQ_FOREACH (current, &owindows, owindows) {
+        Con *ws = con_get_workspace(current->con);
+        if (con_is_internal(ws)) {
+            continue;
+        }
+
+        Output *current_output = get_output_for_con(ws);
+
+        Output *target_output = NULL;
+        TAILQ_FOREACH (uo, &user_output_names, user_output_names) {
+            if (strcasecmp(output_primary_name(current_output), uo->name) == 0) {
+                /* The current output is in the user list */
+                while (true) {
+                    /* This corrupts the outer loop but it is ok since we are
+                     * going to break anyway. */
+                    uo = TAILQ_NEXT(uo, user_output_names);
+                    if (!uo) {
+                        /* We reached the end of the list. We should use the
+                         * first available output that, if it exists, is
+                         * already saved in target_output. */
+                        break;
+                    }
+                    Output *out = get_output_from_string(current_output, uo->name);
+                    if (out) {
+                        DLOG("Found next target for workspace %s from user list: %s\n", ws->name, uo->name);
+                        target_output = out;
+                        break;
+                    }
+                }
+                break;
+            }
+            if (!target_output) {
+                /* The first available output from the list is used in 2 cases:
+                 * 1. When we must wrap around the user list. For example, if
+                 * user specifies outputs A B C and C is `current_output`.
+                 * 2. When the current output is not in the user list. For
+                 * example, user specifies A B C and D is `current_output`.
+                 */
+                DLOG("Found first target for workspace %s from user list: %s\n", ws->name, uo->name);
+                target_output = get_output_from_string(current_output, uo->name);
+            }
+        }
+        if (target_output) {
+            if (move_workspace) {
+                workspace_move_to_output(ws, target_output);
+            } else {
+                con_move_to_output(current->con, target_output, true);
+            }
+            success = true;
+        }
+    }
+
+    while (!TAILQ_EMPTY(&user_output_names)) {
+        uo = TAILQ_FIRST(&user_output_names);
+        free(uo->name);
+        TAILQ_REMOVE(&user_output_names, uo, user_output_names);
+        free(uo);
+    }
+
+    cmd_output->needs_tree_render = success;
+    if (success) {
+        ysuccess(true);
+    } else {
+        yerror("No output matched");
+    }
 }
 
 /*
@@ -1094,36 +1184,6 @@ void cmd_floating(I3_CMD, const char *floating_mode) {
     ysuccess(true);
 }
 
-/*
- * Implementation of 'move workspace to [output] <str>'.
- *
- */
-void cmd_move_workspace_to_output(I3_CMD, const char *name) {
-    DLOG("should move workspace to output %s\n", name);
-
-    HANDLE_EMPTY_MATCH;
-
-    owindow *current;
-    TAILQ_FOREACH (current, &owindows, owindows) {
-        Con *ws = con_get_workspace(current->con);
-        if (con_is_internal(ws)) {
-            continue;
-        }
-
-        Output *current_output = get_output_for_con(ws);
-        Output *target_output = get_output_from_string(current_output, name);
-        if (!target_output) {
-            yerror("Could not get output from string \"%s\"", name);
-            return;
-        }
-
-        workspace_move_to_output(ws, target_output);
-    }
-
-    cmd_output->needs_tree_render = true;
-    ysuccess(true);
-}
-
 /*
  * Implementation of 'split v|h|t|vertical|horizontal|toggle'.
  *
@@ -1661,6 +1721,9 @@ void cmd_restart(I3_CMD) {
     }
     ipc_shutdown(SHUTDOWN_REASON_RESTART, exempt_fd);
     unlink(config.ipc_socket_path);
+    if (current_log_stream_socket_path != NULL) {
+        unlink(current_log_stream_socket_path);
+    }
     /* We need to call this manually since atexit handlers don’t get called
      * when exec()ing */
     purge_zerobyte_logfile();
@@ -1974,6 +2037,35 @@ void cmd_title_format(I3_CMD, const char *format) {
     ysuccess(true);
 }
 
+/*
+ * Implementation of 'title_window_icon <yes|no>' and 'title_window_icon padding <px>'
+ *
+ */
+void cmd_title_window_icon(I3_CMD, const char *enable, int padding) {
+    if (enable != NULL && !boolstr(enable)) {
+        padding = -1;
+    }
+    DLOG("setting window_icon=%d\n", padding);
+    HANDLE_EMPTY_MATCH;
+
+    owindow *current;
+    TAILQ_FOREACH (current, &owindows, owindows) {
+        DLOG("setting window_icon for %p / %s\n", current->con, current->con->name);
+        current->con->window_icon_padding = padding;
+
+        if (current->con->window != NULL) {
+            /* Make sure the window title is redrawn immediately. */
+            current->con->window->name_x_changed = true;
+        } else {
+            /* For windowless containers we also need to force the redrawing. */
+            FREE(current->con->deco_render_params);
+        }
+    }
+
+    cmd_output->needs_tree_render = true;
+    ysuccess(true);
+}
+
 /*
  * Implementation of 'rename workspace [<name>] to <name>'
  *
