@@ -27,22 +27,6 @@ char *current_socketpath = NULL;
 
 TAILQ_HEAD(ipc_client_head, ipc_client) all_clients = TAILQ_HEAD_INITIALIZER(all_clients);
 
-/*
- * Puts the given socket file descriptor into non-blocking mode or dies if
- * setting O_NONBLOCK failed. Non-blocking sockets are a good idea for our
- * IPC model because we should by no means block the window manager.
- *
- */
-static void set_nonblock(int sockfd) {
-    int flags = fcntl(sockfd, F_GETFL, 0);
-    if (flags & O_NONBLOCK) {
-        return;
-    }
-    flags |= O_NONBLOCK;
-    if (fcntl(sockfd, F_SETFL, flags) < 0)
-        err(-1, "Could not set O_NONBLOCK");
-}
-
 static void ipc_client_timeout(EV_P_ ev_timer *w, int revents);
 static void ipc_socket_writeable_cb(EV_P_ struct ev_io *w, int revents);
 
@@ -251,9 +235,9 @@ static void dump_rect(yajl_gen gen, const char *name, Rect r) {
     ystr(name);
     y(map_open);
     ystr("x");
-    y(integer, r.x);
+    y(integer, (int32_t)r.x);
     ystr("y");
-    y(integer, r.y);
+    y(integer, (int32_t)r.y);
     ystr("width");
     y(integer, r.width);
     ystr("height");
@@ -514,6 +498,9 @@ void dump_node(yajl_gen gen, struct Con *con, bool inplace_restart) {
         ystr(con->title_format);
     }
 
+    ystr("window_icon_padding");
+    y(integer, con->window_icon_padding);
+
     if (con->type == CT_WORKSPACE) {
         ystr("num");
         y(integer, con->num);
@@ -573,6 +560,7 @@ void dump_node(yajl_gen gen, struct Con *con, bool inplace_restart) {
         DUMP_PROPERTY("class", class_class);
         DUMP_PROPERTY("instance", class_instance);
         DUMP_PROPERTY("window_role", role);
+        DUMP_PROPERTY("machine", machine);
 
         if (con->window->name != NULL) {
             ystr("title");
@@ -662,6 +650,7 @@ void dump_node(yajl_gen gen, struct Con *con, bool inplace_restart) {
         DUMP_REGEX(instance);
         DUMP_REGEX(window_role);
         DUMP_REGEX(title);
+        DUMP_REGEX(machine);
 
 #undef DUMP_REGEX
         y(map_close);
@@ -1054,6 +1043,17 @@ IPC_HANDLER(get_version) {
     ystr("loaded_config_file_name");
     ystr(current_configpath);
 
+    ystr("included_config_file_names");
+    y(array_open);
+    IncludedFile *file;
+    TAILQ_FOREACH (file, &included_files, files) {
+        if (file == TAILQ_FIRST(&included_files)) {
+            /* Skip the first file, which is current_configpath. */
+            continue;
+        }
+        ystr(file->path);
+    }
+    y(array_close);
     y(map_close);
 
     const unsigned char *payload;
@@ -1236,7 +1236,22 @@ IPC_HANDLER(get_config) {
     y(map_open);
 
     ystr("config");
-    ystr(current_config);
+    IncludedFile *file = TAILQ_FIRST(&included_files);
+    ystr(file->raw_contents);
+
+    ystr("included_configs");
+    y(array_open);
+    TAILQ_FOREACH (file, &included_files, files) {
+        y(map_open);
+        ystr("path");
+        ystr(file->path);
+        ystr("raw_contents");
+        ystr(file->raw_contents);
+        ystr("variable_replaced_contents");
+        ystr(file->variable_replaced_contents);
+        y(map_close);
+    }
+    y(array_close);
 
     y(map_close);
 
@@ -1523,57 +1538,6 @@ ipc_client *ipc_new_client_on_fd(EV_P_ int fd) {
     return client;
 }
 
-/*
- * Creates the UNIX domain socket at the given path, sets it to non-blocking
- * mode, bind()s and listen()s on it.
- *
- */
-int ipc_create_socket(const char *filename) {
-    int sockfd;
-
-    FREE(current_socketpath);
-
-    char *resolved = resolve_tilde(filename);
-    DLOG("Creating IPC-socket at %s\n", resolved);
-    char *copy = sstrdup(resolved);
-    const char *dir = dirname(copy);
-    if (!path_exists(dir))
-        mkdirp(dir, DEFAULT_DIR_MODE);
-    free(copy);
-
-    /* Unlink the unix domain socket before */
-    unlink(resolved);
-
-    if ((sockfd = socket(AF_LOCAL, SOCK_STREAM, 0)) < 0) {
-        perror("socket()");
-        free(resolved);
-        return -1;
-    }
-
-    (void)fcntl(sockfd, F_SETFD, FD_CLOEXEC);
-
-    struct sockaddr_un addr;
-    memset(&addr, 0, sizeof(struct sockaddr_un));
-    addr.sun_family = AF_LOCAL;
-    strncpy(addr.sun_path, resolved, sizeof(addr.sun_path) - 1);
-    if (bind(sockfd, (struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0) {
-        perror("bind()");
-        free(resolved);
-        return -1;
-    }
-
-    set_nonblock(sockfd);
-
-    if (listen(sockfd, 5) < 0) {
-        perror("listen()");
-        free(resolved);
-        return -1;
-    }
-
-    current_socketpath = resolved;
-    return sockfd;
-}
-
 /*
  * Generates a json workspace event. Returns a dynamically allocated yajl
  * generator. Free with yajl_gen_free().
